/*++
 
Copyright (c) Sydus.
 
Module:
 
    ReferenceCounted.h

Authors:
 
    Serge Zloto (szloto) 2006-11-06
 
--*/

#pragma once

#include "base.h"

/**
 * Base class for objects that integrate a reference count for 
 * controlling their lifetime.
 * Instances of derived classes are to be managed by RefCountHandle.
 */
class ReferenceCounted : NonCopyable
{
	public:
		/**
		 * Informs the object that it is being owned once more. This increments 
		 * its reference count
		 */
		void Retain();

		/**
		 * Inform the object that is no longer needed by one of its owners.
		 * The object might delete itself, so the releasing owner must not
		 * use it after this call
		 */
		void Release();

	protected:
		ReferenceCounted();

		virtual ~ReferenceCounted();

	private:
		uint32 iRefCount;
};

inline ReferenceCounted::ReferenceCounted()
	: iRefCount(1)
{
}

inline ReferenceCounted::~ReferenceCounted()
{
}

inline void ReferenceCounted::Retain()
{
	++iRefCount;
}

inline void ReferenceCounted::Release()
{
	if (--iRefCount == 0)
		delete this;
}
